package wiley.streaming.curator;

import static org.junit.Assert.*;

import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.CuratorFrameworkFactory;
import org.apache.curator.framework.api.BackgroundCallback;
import org.apache.curator.framework.api.CuratorEvent;
import org.apache.curator.framework.state.ConnectionState;
import org.apache.curator.framework.state.ConnectionStateListener;
import org.apache.curator.retry.ExponentialBackoffRetry;
import org.apache.zookeeper.data.Stat;
import org.junit.Before;
import org.junit.Test;

public class PathTest {

	CuratorFramework client;
	@Before
	public void setUp() throws Exception {
		client =
				  CuratorFrameworkFactory.builder()
				   .connectString("localhost:2181")
				   .retryPolicy(new ExponentialBackoffRetry(1000,3))
				   .namespace("subspace")
				   .build();
				client.getConnectionStateListenable().addListener(new ConnectionStateListener() {

					public void stateChanged(CuratorFramework arg0, ConnectionState arg1) {
						System.out.println("State Changed: "+arg1);
					}
					
				});
				client.start();
		
	}

	@Test
	public void test() throws Exception {
		Stat stat;
		//Delete the path if it exists already
		if((stat = client.checkExists().forPath("/a/test/path")) != null)
			client.delete()
			.deletingChildrenIfNeeded()
			.withVersion(stat.getVersion())
			.forPath("/a/test/path");

		//Create the path in the foreground
		System.out.println("Creating immediate path: "+client.create()
		  .creatingParentsIfNeeded()
		  .forPath("/a/test/path"));
		
		//Delete the path
		if((stat = client.checkExists().forPath("/a/test/path")) != null)
			client.delete()
			.deletingChildrenIfNeeded()
			.withVersion(stat.getVersion())
			.forPath("/a/test/path");
		client.create()
		.creatingParentsIfNeeded()
		.inBackground(new BackgroundCallback() {

		public void processResult(CuratorFramework arg0, CuratorEvent arg1)
		  throws Exception {
		    System.out.println("Path Created: "+arg1);
		  }


		})
		.forPath("/a/test/path");
		Thread.sleep(1000);
		
	}

}
