package wiley.streaming.camus;

import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.compress.CompressionCodec;
import org.apache.hadoop.io.compress.DefaultCodec;
import org.apache.hadoop.mapreduce.RecordWriter;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.output.FileOutputCommitter;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.SequenceFileOutputFormat;
import org.apache.hadoop.util.ReflectionUtils;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.node.ObjectNode;

import com.linkedin.camus.coders.CamusWrapper;
import com.linkedin.camus.etl.IEtlKey;
import com.linkedin.camus.etl.RecordWriterProvider;
import com.linkedin.camus.etl.kafka.mapred.EtlMultiOutputFormat;

public class JSONRecordWriter implements RecordWriterProvider {

	@Override
	public RecordWriter<IEtlKey, CamusWrapper> getDataRecordWriter(
			TaskAttemptContext context, String filename, CamusWrapper arg2,
			FileOutputCommitter comitter) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		Path file = new Path(comitter.getWorkPath(),EtlMultiOutputFormat.getUniqueFile(context, filename, getFilenameExtension()));
		CompressionCodec codec = null;
		SequenceFile.CompressionType type = SequenceFile.CompressionType.NONE;
		if(FileOutputFormat.getCompressOutput(context)) {
			type = SequenceFileOutputFormat.getOutputCompressionType(context);
			Class<?> klass = SequenceFileOutputFormat.getOutputCompressorClass(context, DefaultCodec.class);
			codec = (CompressionCodec) ReflectionUtils.newInstance(klass, conf);
		}
		final SequenceFile.Writer writer = SequenceFile.createWriter(
				file.getFileSystem(conf), 
				conf, 
				file, 
				Text.class, Text.class, 
				type, 
				codec);
		return new RecordWriter<IEtlKey,CamusWrapper>() {
			Text key   = new Text();
			Text value = new Text();
			

			@Override
			public void close(TaskAttemptContext arg0) throws IOException,
					InterruptedException {
				writer.close();
			}

			@Override
			public void write(IEtlKey arg0, CamusWrapper arg1)
					throws IOException, InterruptedException {
				key.set(arg0.getTopic()+"\t"+arg0.getPartition()+"\t"+arg0.getOffset());
				if(arg1.getRecord() instanceof ObjectNode) {
					ObjectNode node = (ObjectNode)arg1.getRecord();
					value.set(node.toString());
				}
			}
			
		};
	}

	@Override
	public String getFilenameExtension() {
		return ".json";
	}

}
