package wiley.streaming.flume;

import java.nio.charset.Charset;

import org.apache.flume.Context;
import org.apache.flume.EventDrivenSource;
import org.apache.flume.channel.ChannelProcessor;
import org.apache.flume.conf.Configurable;
import org.apache.flume.event.EventBuilder;
import org.apache.flume.source.AbstractSource;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;
import redis.clients.jedis.JedisPubSub;

public class RedisSource extends AbstractSource implements Configurable,
		EventDrivenSource {

	String           redisHost;
	int              redisPort;
	String           redisChannel;
	Jedis            jedis;
	
	public void configure(Context context) {
		redisHost    = context.getString("redis-host", "localhost");
		redisPort    = context.getInteger("redis-port", 6379);
		redisChannel = context.getString("redis-channel", "flume"); 
	}

	
	ChannelProcessor processor;
	JedisPubSub      pubSub = new JedisPubSub() {

		@Override
		public void onMessage(String arg0, String arg1) {
			processor.processEvent(EventBuilder.withBody(arg1, Charset.defaultCharset()));
		}

		@Override
		public void onPMessage(String arg0, String arg1, String arg2) {}

		@Override
		public void onPSubscribe(String arg0, int arg1) {}

		@Override
		public void onPUnsubscribe(String arg0, int arg1) {}

		@Override
		public void onSubscribe(String arg0, int arg1) {}

		@Override
		public void onUnsubscribe(String arg0, int arg1) {}
		
	};
	
	@Override
	public synchronized void start() {
		super.start();
		
		processor = getChannelProcessor();
		
		JedisPool pool = new JedisPool(new JedisPoolConfig(),redisHost,redisPort);
		jedis = pool.getResource();
		new Thread(new Runnable() {

			public void run() {
				jedis.subscribe(pubSub, redisChannel);
			}
			
		}).start();
		
	}

	@Override
	public synchronized void stop() {
		pubSub.unsubscribe();
		super.stop();
	}
	

}
