package wiley.streaming.nn;

public abstract class Activation {
	
	public abstract double f(double x);
	public abstract double df(double x);

	public static Activation logit(final double k) {
		return new Activation() {
			@Override
			public double f(double x) {
				return 1.0/(1.0 + Math.exp(-k*x));
			}

			@Override
			public double df(double x) {
				x = f(x);
				return k*x*(1.0-x);
			}
		};
	}
	public static Activation sigmoid(double k) {
		return logit(k);
	}
	public static Activation logit() {
		return logit(1.0);
	}
	public static Activation sigmoid() {
		return logit();
	}
	
	public static Activation tanh() {
		return new Activation() {
			@Override
			public double f(double x) {
				return Math.tanh(x);
			}

			@Override
			public double df(double x) {
				x = f(x);
				return 1.0 - x*x;
			}
		};
	}
}
