package wiley.streaming.ols;

import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.QRDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.apache.commons.math3.stat.regression.OLSMultipleLinearRegression;

public class LinearModel {

	double[] B;
	
	public LinearModel(double[] B) {
		this.B = B;
	}
	
	public double y(double[] x) {
		double y = 0;
		for(int i=0;i<B.length;i++) y += B[i]*x[i];
		return y;
	}
	
	public double error(double[] y,double[][] x) {
		double error = 0.0;
		for(int i=0;i<y.length;i++) {
			double diff = y[i] - y(x[i]);
			error += diff*diff;
		}
		return error;
	}
	
	public void fit(double[] y,double[][] x) {
		RealMatrix X = new Array2DRowRealMatrix(x);
		RealVector Y = new ArrayRealVector(y);
		B = (new QRDecomposition(X)).getSolver().solve(Y).toArray();
	}
	
	public void fitOLS(double[] y,double[][] x) {
		OLSMultipleLinearRegression ols = new OLSMultipleLinearRegression();
		ols.newSampleData(y, x);
		B = ols.estimateRegressionParameters();
	}
}
