package wiley.realtime.sampler;

import wiley.realtime.Distributions;
import cern.jet.math.Arithmetic;
import cern.jet.random.engine.MersenneTwister;
import cern.jet.stat.Gamma;

public class Distribution {
	MersenneTwister rng = new MersenneTwister();
	
	public double nextUniform(double a,double b) {
		return (b-a)*rng.nextDouble() + a;
	}
	
	public double nextExponential(double p) {
		return -Math.log(rng.nextDouble())/p;
	}
	
	public int nextPoisson(double p) {
		int    k = 0;
		double c = 0, u = rng.nextDouble()/Math.exp(-p);
		while(true) {
			double x = c + Math.pow(p, k)/Arithmetic.factorial(k);
			if(x > u) return k;
			c += x;
			k++;
		}
	}
	
	
	protected double gamma_f(double x,double a) {
		double L = Math.sqrt(2*a - 1);
		double y = Math.log(4) - a + (L+a)*Math.log(a) 
				+ (L-1)*Math.log(x) - Gamma.logGamma(a) 
				- 2*Math.log(Math.pow(a, L) + Math.pow(x, L));
		return Math.exp(y);
	}
	
	protected double nextGammaF(double a) {
		double L = Math.sqrt(2*a - 1);
		double u = rng.nextDouble();
		return a*Math.pow(u/(1-u), 1/L);
	}
	
	public double nextGamma(double k,double p) {
		while(true) {
			double x = nextGammaF(k);
			if(rng.nextDouble() <= gamma_f(x,k)/Distributions.dgamma(x, k, 1)) {
				return x/p;
			}
		}
	}
	
	public double nextChiSq(double k) { return nextGamma(k/2,0.5); }
	
	public double nextBeta(double a,double b) {
		double x = nextGamma(a,1);
		double y = nextGamma(b,1);
		return x/(x+y);
	}
	
	double  z2;
	boolean ready = false;
	
	public double nextZNormal() {
		if(ready) { ready = false;return z2; }
		double theta = nextUniform(0,2*Math.PI);
		double r     = Math.sqrt(nextExponential(0.5)); 
		z2 = r*Math.sin(theta);ready = true;
		return r*Math.cos(theta);
	}
	
	public double nextNormal(double mu,double sig) {
		return mu + sig*nextZNormal();
	}
	
	
}
